Package: MNP
Version: 3.1-5
Title: Fitting the Multinomial Probit Model
Authors@R: c(
  person("Kosuke", "Imai", , "imai@harvard.edu", c("aut", "cre")),
  person("David", "van Dyk", , "dvandyk@imperial.ac.uk", c("aut")),
  person("Hubert", "Jin", , "hubertj@princeton.edu", c("ctb"))
  )
Maintainer: Kosuke Imai <imai@harvard.edu>
Depends: R (>= 2.1), stats, utils
Suggests: testthat
Description: Fits the Bayesian multinomial probit model via Markov chain
 Monte Carlo.  The multinomial probit model is often used to analyze 
 the discrete choices made by individuals recorded in survey data. 
 Examples where the multinomial probit model may be useful include the 
 analysis of product choice by consumers in market research and the 
 analysis of candidate or party choice by voters in electoral studies.  
 The MNP package can also fit the model with different choice sets for 
 each individual, and complete or partial individual choice orderings 
 of the available alternatives from the choice set. The estimation is
 based on the efficient marginal data augmentation algorithm that is 
 developed by Imai and van Dyk (2005). "A Bayesian Analysis of the 
 Multinomial Probit Model Using the Data Augmentation." Journal of 
 Econometrics, Vol. 124, No. 2 (February), pp. 311-334. 
 <doi:10.1016/j.jeconom.2004.02.002>  Detailed examples are given in 
 Imai and van Dyk (2005). "MNP: R Package for Fitting the Multinomial 
 Probit Model."  Journal of Statistical Software, Vol. 14, No. 3 (May), 
 pp. 1-32. <doi:10.18637/jss.v014.i03>.
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
Encoding: UTF-8
URL: https://github.com/kosukeimai/MNP
BugReports: https://github.com/kosukeimai/MNP/issues
NeedsCompilation: yes
RoxygenNote: 7.3.1
Packaged: 2024-06-20 02:44:05 UTC; kosukeimai
Author: Kosuke Imai [aut, cre],
  David van Dyk [aut],
  Hubert Jin [ctb]
Repository: CRAN
Date/Publication: 2024-06-20 10:20:25 UTC

# Generated by roxygen2: do not edit by hand

S3method(coef,mnp)
S3method(predict,mnp)
S3method(print,mnp)
S3method(print,summary.mnp)
S3method(summary,mnp)
S3method(vcov,mnp)
export(mnp)
useDynLib(MNP, .registration = TRUE)

#' Extract Multinomial Probit Model Coefficients
#'
#' \code{coef.mnp} is a function which extracts multinomial probit model
#' coefficients from ojbects returned by \code{mnp}. \code{coefficients.mnp} is
#' an alias for it. \code{coef} method for class \code{mnp}.
#'
#'
#' @param object An output object from \code{mnp}.
#' @param subset A scalar or a numerical vector specifying the row number(s) of
#' \code{param} in the output object from \code{mnp}. If specified, the
#' posterior draws of coefficients for those rows are extracted. The default is
#' \code{NULL} where all the posterior draws are extracted.
#' @param ... further arguments passed to or from other methods.
#' @return \code{coef.mnp} returns a matrix (when a numerical vector or
#' \code{NULL} is specified for \code{subset} argument) or a vector (when a
#' scalar is specified for \code{subset} arugment) of multinomila probit model
#' coefficients.
#' @author Kosuke Imai, Department of Government and Department of Statistics, Harvard University
#' \email{kimai@Princeton.Edu}
#' @seealso \code{mnp}, \code{vcov.mnp};
#' @keywords methods
#' @exportS3Method coef mnp
coef.mnp <- function(object, subset = NULL, ...) {
  param <- object$param
  p <- object$n.alt
  n.cov <- ncol(param) - p*(p-1)/2
  n <- nrow(param)
  if (is.null(subset))
    return(param[,1:n.cov])
  else if (subset > n)
    stop(paste("invalid input for `subset.' only", nrow(param), "draws are stored."))
  else
    return(param[subset, 1:n.cov])
}

#' Posterior Prediction under the Bayesian Multinomial Probit Models
#'
#' Obtains posterior predictions under a fitted (Bayesian) multinomial probit
#' model. \code{predict} method for class \code{mnp}.
#'
#' The posterior predictive values are computed using the Monte Carlo sample
#' stored in the \code{mnp} output (or other sample if \code{newdraw} is
#' specified). Given each Monte Carlo sample of the parameters and each vector
#' of predictor variables, we sample the vector-valued latent variable from the
#' appropriate multivariate Normal distribution. Then, using the sampled
#' predictive values of the latent variable, we construct the most preferred
#' choice as well as the ordered preferences. Averaging over the Monte Carlo
#' sample of the preferred choice, we obtain the predictive probabilities of
#' each choice being most preferred given the values of the predictor
#' variables.  Since the predictive values are computed via Monte Carlo
#' simulations, each run may produce somewhat different values. The computation
#' may be slow if predictions with many values of the predictor variables are
#' required and/or if a large Monte Carlo sample of the model parameters is
#' used. In either case, setting \code{verbose = TRUE} may be helpful in
#' monitoring the progress of the code.
#'
#' @param object An output object from \code{mnp}.
#' @param newdata An optional data frame containing the values of the predictor
#' variables. Predictions for multiple values of the predictor variables can be
#' made simultaneously if \code{newdata} has multiple rows. The default is the
#' original data frame used for fitting the model.
#' @param newdraw An optional matrix of MCMC draws to be used for posterior
#' predictions. The default is the original MCMC draws stored in \code{object}.
#' @param n.draws The number of additional Monte Carlo draws given each MCMC
#' draw of coefficients and covariance matrix. The specified number of latent
#' variables will be sampled from the multivariate normal distribution, and the
#' quantities of interest will be calculated by averaging over these draws.
#' This will be particularly useful calculating the uncertainty of predicted
#' probabilities. The default is \code{1}.
#' @param type The type of posterior predictions required. There are four
#' options: \code{type = "prob"} returns the predictive probabilities of being
#' the most preferred choice among the choice set.  \code{type = "choice"}
#' returns the Monte Carlo sample of the most preferred choice, and \code{type
#' = "order"} returns the Monte Carlo sample of the ordered preferences,
#' @param verbose logical. If \code{TRUE}, helpful messages along with a
#' progress report on the Monte Carlo sampling from the posterior predictive
#' distributions are printed on the screen. The default is \code{FALSE}.
#' @param ... additional arguments passed to other methods.
#' @return \code{predict.mnp} yields a list of class
#' \code{predict.mnp} containing at least one of the following elements:
#' \item{o}{A three dimensional array of the Monte Carlo sample from the posterior predictive
#' distribution of the ordered preferences. The first dimension corresponds to
#' the rows of \code{newdata} (or the original data set if \code{newdata} is
#' left unspecified), the second dimension corresponds to the alternatives in
#' the choice set, and the third dimension indexes the Monte Carlo sample. If
#' \code{n.draws} is greater than 1, then each entry will be an average over
#' these additional draws.  }
#' \item{p}{A two or three dimensional array of the
#' posterior predictive probabilities for each alternative in the choice set
#' being most preferred. The first demension corresponds to the rows of
#' \code{newdata} (or the original data set if \code{newdata} is left
#' unspecified), the second dimension corresponds to the alternatives in the
#' choice set, and the third dimension (if it exists) indexes the Monte Carlo
#' sample. If \code{n.draws} is greater than 1, then the third diemsion exists
#' and indexes the Monte Carlo sample.  }
#' \item{y}{A matrix of the Monte Carlo
#' sample from the posterior predictive distribution of the most preferred
#' choice. The first dimension correspond to the rows of \code{newdata} (or the
#' original data set if \code{newdata} is left unspecified), and the second
#' dimension indexes the Monte Carlo sample. \code{n.draws} will be set to 1
#' when computing this quantity of interest.  }
#' \item{x}{A matrix of covariates used for prediction}
#' @author Kosuke Imai, Department of Government and Department of Statistics, Harvard University
#' \email{imai@Harvard.Edu}
#' @seealso \code{mnp}
#' @keywords methods
#' @method predict mnp
#' @exportS3Method predict mnp
predict.mnp <- function(object, newdata = NULL, newdraw = NULL, n.draws = 1,
                        type = c("prob", "choice", "order"),
                        verbose = FALSE, ...){

  if (NA %in% match(type, c("prob", "choice", "order")))
    stop("Invalid input for `type'.")
  if (n.draws < 1)
    stop("Invalid input for `n.draws'.")

  p <- object$n.alt
  if (is.null(newdraw))
    param <- object$param
  else
    param <- newdraw
  n.cov <- ncol(coef(object))
  coef <- param[,1:n.cov]
  n.mcmc <- nrow(coef)
  cov <- param[,(n.cov+1):ncol(param)]

  ## get X matrix ready
  if (is.null(newdata))
    x <- object$x
  else {
    call <- object$call
    x <- xmatrix.mnp(stats::as.formula(call$formula), data = newdata,
                     choiceX = call$choiceX,
                     cXnames = eval(call$cXnames),
                     base = object$base, n.dim = p-1,
                     lev = object$alt, MoP = is.matrix(object$y),
                     verbose = FALSE, extra = FALSE)
    if (nrow(x) > 1)
      x <- as.matrix(x[apply(is.na(x), 1, sum)==0,])
    else if (sum(is.na(x))>0)
      stop("Invalid input for `newdata'.")
  }
  n.obs <- nrow(x)
  if (verbose) {
    if (n.obs == 1)
      cat("There is one observation to predict. Please wait...\n")
    else
      cat("There are", n.obs, "observations to predict. Please wait...\n")
  }

  alt <- object$alt
  if (object$base != alt[1])
    alt <- c(object$base, alt[1:(length(alt)-1)])

  res <- .C("predictP", as.double(x), as.integer(n.obs),
            as.double(coef), as.double(cov), as.integer(p-1),
            as.integer(n.cov), as.integer(n.mcmc),
            as.integer(n.draws), as.integer(verbose),
            prob = if (n.draws > 1) double(n.obs*n.mcmc*p)
                   else double(n.obs*p),
            choice = double(n.obs*n.mcmc),
            order = double(n.obs*n.mcmc*p),
            PACKAGE = "MNP")

  ans <- list()
  if ("choice" %in% type)
    ans$y <- matrix(res$choice, ncol=n.mcmc, nrow = n.obs,
                    byrow = TRUE, dimnames = list(rownames(x), 1:n.mcmc))
  else
    ans$y <- NULL
  if ("order" %in% type)
    ans$o <- aperm(array(res$order, dim=c(p, n.mcmc, n.obs),
                         dimnames = list(alt, 1:n.mcmc,
                           rownames(x))), c(3,1,2))
  else
    ans$o <- NULL
  if ("prob" %in% type)
    if (n.draws > 1)
      ans$p <- aperm(array(res$prob, dim = c(p, n.mcmc, n.obs),
                           dimnames = list(alt, 1:n.mcmc,
                             rownames(x))), c(3,1,2))
    else
      ans$p <- matrix(res$prob, nrow = n.obs, ncol = p, byrow = TRUE,
                      dimnames = list(rownames(x), alt))
  else
    ans$p <- NULL

  ans$x <- x
  class(ans) <- "predict.mnp"
  return(ans)
}

#' Print the summary of the results for the Multinomial Probit Models
#'
#' \code{summary} print method for class \code{mnp}.
#'
#' @param x An object of class \code{summary.mnp}.
#' @param digits the number of significant digits to use when printing.
#' @param ... further arguments passed to or from other methods.
#' @author Kosuke Imai, Department of Politics, Princeton University
#' \email{kimai@Princeton.Edu}
#' @seealso \code{mnp}
#' @keywords methods
#' @exportS3Method print summary.mnp
print.summary.mnp <- function(x, digits = max(3, getOption("digits") - 3), ...) {

  cat("\nCall:\n")
  cat(paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n",
      sep = "")

  cat("\nCoefficients:\n")
  stats::printCoefmat(x$coef.table, digits = digits, na.print = "NA", ...)

  cat("\nCovariances:\n")
  stats::printCoefmat(x$cov.table, digits = digits, na.print = "NA", ...)

  cat("\nBase category:", x$base)
  cat("\nNumber of alternatives:", x$n.alt)
  cat("\nNumber of observations:", x$n.obs)
  cat("\nNumber of estimated parameters:", x$n.param)
  cat("\nNumber of stored MCMC draws:", x$n.draws)
  cat("\n\n")
  invisible(x)
}

#' Summarizing the results for the Multinomial Probit Models
#'
#' \code{summary} method for class \code{mnp}.
#'
#'
#' @param object An output object from \code{mnp}.
#' @param CI A 2 dimensional vector of lower and upper bounds for the credible
#' intervals used to summarize the results. The default is the equal tail 95
#' percent credible interval.
#' @param ... further arguments passed to or from other methods.
#' @return \code{summary.mnp} yields an object of class \code{summary.mnp}
#' containing the following elements: \item{call}{The call from \code{mnp}.}
#' \item{n.alt}{The total number of alternatives.} \item{base}{The base
#' category used for fitting.} \item{n.obs}{The number of observations.}
#' \item{n.param}{The number of estimated parameters.} \item{n.draws}{The
#' number of Gibbs draws used for the summary.} \item{coef.table}{The summary
#' of the posterior distribution of the coefficients. } \item{cov.table}{The
#' summary of the posterior distribution of the covariance matrix.} This object
#' can be printed by \code{print.summary.mnp}
#' @author Kosuke Imai, Department of Government and Department of Statistics, Harvard University
#' \email{imai@Harvard.Edu}
#' @seealso \code{mnp}
#' @keywords methods
#' @exportS3Method summary mnp
summary.mnp <- function(object, CI=c(2.5, 97.5),...){

  p <- object$n.alt
  param <- object$param
  n.cov <- ncol(param) - p*(p-1)/2
  n.draws <- nrow(param)
  param.table <- cbind(apply(param, 2, mean), apply(param, 2, stats::sd),
                       apply(param, 2, stats::quantile, min(CI)/100),
                       apply(param, 2, stats::quantile, max(CI)/100))
  colnames(param.table) <- c("mean", "std.dev.", paste(min(CI), "%", sep=""),
                             paste(max(CI), "%", sep=""))
  rownames(param.table) <- colnames(param)

  ans <- list(call = object$call, base = object$base, n.alt=p,
              n.obs = if(is.matrix(object$y)) nrow(object$y) else length(object$y),
              n.param = ncol(param)-1, n.draws = n.draws,
              coef.table= if(n.cov > 1) param.table[1:n.cov,]
              else matrix(param.table[1,], nrow=1,
                          dimnames=list(rownames(param.table)[1], colnames(param.table))),
              cov.table=param.table[(n.cov+1):ncol(param),])
  class(ans) <- "summary.mnp"
  return(ans)
}

#' @exportS3Method print mnp
print.mnp <- function (x, digits = max(3, getOption("digits") - 3), ...)
  {
    cat("\nCall:\n", deparse(x$call), "\n\n", sep = "")
    param <- apply(x$param, 2, mean)
    if (length(param)) {
      cat("Parameter estimates (posterior means):\n")
      print.default(format(param, digits = digits), print.gap = 2,
                    quote = FALSE)
    }
    else cat("No parameter estimates\n")
    cat("\n")
    invisible(x)
  }

#' Voters' Preferences of Political Parties in Japan (1995)
#' 
#' This dataset gives voters' preferences of political parties in Japan on the
#' 0 (least preferred) - 100 (most preferred) scale.  It is based on the 1995
#' survey data of 418 individual voters.  The data also include the sex,
#' education level, and age of the voters. The survey allowed voters to choose
#' among four parties: Liberal Democratic Party (LDP), New Frontier Party
#' (NFP), Sakigake (SKG), and Japanese Communist Party (JCP).
#' 
#' 
#' @name japan
#' @docType data
#' @format A data frame containing the following 7 variables for 418
#' observations.
#' 
#' \tabular{llll}{ LDP \tab numeric \tab preference for Liberal Democratic
#' Party \tab 0 - 100 \cr NFP \tab numeric \tab preference for New Frontier
#' Party \tab 0 - 100 \cr SKG \tab numeric \tab preference for Sakigake \tab 0
#' - 100 \cr JCP \tab numeric \tab preference for Japanese Communist Party \tab
#' 0 - 100 \cr gender \tab factor \tab gender of each voter \tab \code{male} or
#' \code{female} \cr education \tab numeric \tab levels of education for each
#' voter \tab \cr age \tab numeric \tab age of each voter \tab }
#' @keywords datasets
NULL

#' Detergent Brand Choice
#' 
#' This dataset gives the laundry detergent brand choice by households and the
#' price of each brand.
#' 
#' 
#' @name detergent
#' @docType data
#' @format A data frame containing the following 7 variables and 2657
#' observations.
#' 
#' \tabular{lll}{ choice \tab factor \tab a brand chosen by each household\cr
#' TidePrice \tab numeric \tab log price of Tide\cr WiskPrice \tab numeric \tab
#' log price of Wisk\cr EraPlusPrice \tab numeric \tab log price of EraPlus\cr
#' SurfPrice \tab numeric \tab log price of Surf\cr SoloPrice \tab numeric \tab
#' log price of Solo\cr AllPrice \tab numeric \tab log price of All\cr }
#' @references Chintagunta, P. K. and Prasad, A. R. (1998) \dQuote{An Empirical
#' Investigation of the `Dynamic McFadden' Model of Purchase Timing and Brand
#' Choice: Implications for Market Structure}. \emph{Journal of Business and
#' Economic Statistics} vol. 16 no. 1 pp.2-12.
#' @keywords datasets
NULL

".onAttach" <- function(lib, pkg) {
  mylib <- dirname(system.file(package = pkg))
  title <- utils::packageDescription(pkg, lib.loc = mylib)$Title
  ver <- utils::packageDescription(pkg, lib.loc = mylib)$Version
  author <- utils::packageDescription(pkg, lib.loc = mylib)$Author
  packageStartupMessage(pkg, ": ", title, "\nVersion: ", ver, "\nAuthors: ", author, "\n")
}

ymatrix.mnp <- function(data, base=NULL, extra=FALSE) {
  ## checking and formatting Y
  Y <- stats::model.response(data)
  if (is.matrix(Y)) { # Multinomial ordered Probit model
    for (i in 1:nrow(Y))
      Y[i,] <- match(Y[i,], sort(unique(Y[i,]))) - 1
    p <- ncol(Y)
    lev <- colnames(Y)
    MoP <- TRUE
    if(!is.null(base))
      stop("Error: The last column of the response matrix must be the base category.\n No need to specify `base.'")
    base <- lev[p]
  } else { # standard Multinomial Probit model
    Y <- as.factor(Y)
    lev <- levels(Y)
    if (!is.null(base))
      if (base %in% lev) {
        Y <- stats::relevel(Y, ref = base)
        lev <- levels(Y)
      } else {
        stop(paste("Error: `base' does not exist in the response variable."))
      }
    base <- lev[1]
    counts <- table(Y)
    if (any(counts == 0)) {
      warning(paste("group(s)", paste(lev[counts == 0], collapse = " "), "are empty"))
      Y <- factor(Y, levels  = lev[counts > 0])
      lev <- lev[counts > 0]
    }
    p <- length(lev)
    Y <- as.matrix(unclass(Y)) - 1
    MoP <- FALSE
  }
  if(extra)
    return(list(Y=Y, MoP=MoP, lev=lev, p=p, base=base))
  else
    return(Y)
}

#' Fitting the Multinomial Probit Model via Markov chain Monte Carlo
#'
#' \code{mnp} is used to fit (Bayesian) multinomial probit model via Markov
#' chain Monte Carlo.  \code{mnp} can also fit the model with different choice
#' sets for each observation, and complete or partial ordering of all the
#' available alternatives. The computation uses the efficient marginal data
#' augmentation algorithm that is developed by Imai and van Dyk (2005a).
#'
#' To fit the multinomial probit model when only the most preferred choice is
#' observed, use the syntax for the formula, \code{y ~ x1 + x2}, where \code{y}
#' is a factor variable indicating the most preferred choice and \code{x1} and
#' \code{x2} are individual-specific covariates. The interactions of
#' individual-specific variables with each of the choice indicator variables
#' will be fit.
#'
#' To specify choice-specific covariates, use the syntax,
#' \code{choiceX=list(A=cbind(z1, z2), B=cbind(z3, z4), C=cbind(z5, z6))},
#' where \code{A}, \code{B}, and \code{C} represent the choice names of the
#' response variable, and \code{z1} and \code{z2} are each vectors of length
#' \eqn{n} that record the values of the two choice-specific covariates for
#' each individual for choice A, likewise for \code{z3}, \eqn{\ldots},
#' \code{z6}. The corresponding variable names via \code{cXnames=c("price",
#' "quantity")} need to be specified, where \code{price} refers to the
#' coefficient name for \code{z1}, \code{z3}, and \code{z5}, and
#' \code{quantity} refers to that for \code{z2}, \code{z4}, and \code{z6}.
#'
#' If the choice set varies from one observation to another, use the syntax,
#' \code{cbind(y1, y2, y3) ~ x1 + x2}, in the case of a three choice problem,
#' and indicate unavailable alternatives by \code{NA}. If only the most
#' preferred choice is observed, \code{y1}, \code{y2}, and \code{y3} are
#' indicator variables that take on the value one for individuals who prefer
#' that choice and zero otherwise. The last column of the response matrix,
#' \code{y3} in this particular example syntax, is used as the base category.
#'
#' To fit the multinomial probit model when the complete or partial ordering of
#' the available alternatives is recorded, use the same syntax as when the
#' choice set varies (i.e., \code{cbind(y1, y2, y3, y4) ~ x1 + x2}). For each
#' observation, all the available alternatives in the response variables should
#' be numerically ordered in terms of preferences such as \code{1 2 2 3}. Ties
#' are allowed. The missing values in the response variable should be denoted
#' by \code{NA}. The software will impute these missing values using the
#' specified covariates. The resulting uncertainty estimates of the parameters
#' will properly reflect the amount of missing data. For example, we expect the
#' standard errors to be larger when there is more missing data.
#'
#' @aliases mnp MNP
#' @param formula A symbolic description of the model to be fit specifying the
#' response variable and covariates. The formula should not include the
#' choice-specific covariates. Details and specific examples are given below.
#' @param data An optional data frame in which to interpret the variables in
#' \code{formula} and \code{choiceX}. The default is the environment in which
#' \code{mnp} is called.
#' @param choiceX An optional list containing a matrix of choice-specific
#' covariates for each category. Details and examples are provided below.
#' @param cXnames A vector of the names for the choice-specific covariates
#' specified in \code{choiceX}. The details and examples are provided below.
#' @param base The name of the base category. For the standard multinomial
#' probit model, the default is the lowest level of the response variable. For
#' the multinomial probit model with ordered preferences, the default base
#' category is the last column in the matrix of response variables.
#' @param latent logical. If \code{TRUE}, then the latent variable W will be
#' returned. See Imai and van Dyk (2005) for the notation. The default is
#' \code{FALSE}.
#' @param invcdf logical. If \code{TRUE}, then the inverse cdf method is used
#' for truncated normal sampling. If \code{FALSE}, then the rejection sampling
#' method is used. The default is \code{FALSE}.
#' @param trace logical. If \code{TRUE}, then the trace of the variance
#' covariance matrix is set to a constant (here, it is equal to \code{n.dim})
#' instead of setting its first diagonal element to 1.  The former avoids the
#' arbitrariness of fixing one particular diagonal element in order to achieve
#' identification (see Burgette and Nordheim, 2009).
#' @param n.draws A positive integer. The number of MCMC draws. The default is
#' \code{5000}.
#' @param p.var A positive definite matrix. The prior variance of the
#' coefficients.  A scalar input can set the prior variance to the diagonal
#' matrix whose diagonal element is equal to that value. The default is
#' \code{"Inf"}, which represents an improper noninformative prior distribution
#' on the coefficients.
#' @param p.df A positive integer greater than \code{n.dim-1}. The prior
#' degrees of freedom parameter for the covariance matrix. The default is
#' \code{n.dim+1}, which is equal to the total number of alternatives.
#' @param p.scale A positive definite matrix.  When \code{trace = FALSE}, its
#' first diagonal element is set to \code{1} if it is not equal to 1 already.
#' The prior scale matrix for the covariance matrix. A scalar input can be used
#' to set the scale matrix to a diagonal matrix with diagonal elements equal to
#' the scalar input value. The default is \code{1}.
#' @param coef.start A vector. The starting values for the coefficients.  A
#' scalar input sets the starting values for all the coefficients equal to that
#' value.  The default is \code{0}.
#' @param cov.start A positive definite matrix. When \code{trace = FALSE}, its
#' first diagonal element is set to \code{1} if it is not equal to 1 already.
#' The starting values for the covariance matrix. A scalar input can be used to
#' set the starting value to a diagonal matrix with diagonal elements equal to
#' the scalar input value. The default is \code{1}.
#' @param burnin A positive integer. The burnin interval for the Markov chain;
#' i.e., the number of initial Gibbs draws that should not be stored. The
#' default is \code{0}.
#' @param thin A positive integer. The thinning interval for the Markov chain;
#' i.e., the number of Gibbs draws between the recorded values that are
#' skipped. The default is \code{0}.
#' @param verbose logical. If \code{TRUE}, helpful messages along with a
#' progress report of the Gibbs sampling are printed on the screen. The default
#' is \code{FALSE}.
#' @return An object of class \code{mnp} containing the following elements:
#' \item{param}{A matrix of the Gibbs draws for each parameter; i.e., the
#' coefficients and covariance matrix. For the covariance matrix, the elements
#' on or above the diagonal are returned.  }
#' \item{call}{The matched call.}
#' \item{x}{The matrix of covariates.}
#' \item{y}{The vector or matrix of the
#' response variable.}
#' \item{w}{The three dimensional array of the latent
#' variable, W. The first dimension represents the alternatives, and the second
#' dimension indexes the observations. The third dimension represents the Gibbs
#' draws. Note that the latent variable for the base category is set to 0, and
#' therefore omitted from the output.}
#' \item{alt}{The names of alternatives.}
#' \item{n.alt}{The total number of alternatives.}
#' \item{base}{The base
#' category used for fitting.}
#' \item{invcdf}{The value of
#' \code{invcdf}.}
#' \item{p.var}{The prior variance for the coefficients.}
#' \item{p.df}{The prior
#' degrees of freedom parameter for the covariance matrix.}
#' \item{p.scale}{The
#' prior scale matrix for the covariance matrix.}
#' \item{burnin}{The number of
#' initial burnin draws.}
#' \item{thin}{The thinning interval.}
#' @author Kosuke Imai, Department of Government and Department of Statistics, Harvard University
#' \email{imai@@Harvard.Edu}, \url{https://imai.fas.harvard.edu}; David A. van
#' Dyk, Statistics Section, Department of Mathematics, Imperial College London.
#' @seealso \code{coef.mnp}, \code{vcov.mnp}, \code{predict.mnp},
#' \code{summary.mnp};
#' @references Imai, Kosuke and David A. van Dyk. (2005a) \dQuote{A Bayesian
#' Analysis of the Multinomial Probit Model Using the Marginal Data
#' Augmentation,} \emph{Journal of Econometrics}, Vol. 124, No. 2 (February),
#' pp.311-334.
#'
#' Imai, Kosuke and David A. van Dyk. (2005b) \dQuote{MNP: R Package for
#' Fitting the Multinomial Probit Models,} \emph{Journal of Statistical
#' Software}, Vol. 14, No. 3 (May), pp.1-32.
#'
#' Burgette, L.F. and E.V. Nordheim. (2009).  \dQuote{An alternate identifying
#' restriction for the Bayesian multinomial probit model,} \emph{Technical
#' report}, Department of Statistics, University of Wisconsin, Madison.
#' @keywords models
#' @useDynLib MNP, .registration = TRUE
#' @examples
#'
#' ###
#' ### NOTE: this example is not fully analyzed. In particular, the
#' ### convergence has not been assessed. A full analysis of these data
#' ### sets appear in Imai and van Dyk (2005b).
#' ###
#'
#' ## load the detergent data
#' data(detergent)
#' ## run the standard multinomial probit model with intercepts and the price
#' res1 <- mnp(choice ~ 1, choiceX = list(Surf=SurfPrice, Tide=TidePrice,
#'                                        Wisk=WiskPrice, EraPlus=EraPlusPrice,
#'                                        Solo=SoloPrice, All=AllPrice),
#'             cXnames = "price", data = detergent, n.draws = 100, burnin = 10,
#'             thin = 3, verbose = TRUE)
#' ## summarize the results
#' summary(res1)
#' ## calculate the quantities of interest for the first 3 observations
#' pre1 <- predict(res1, newdata = detergent[1:3,])
#'
#' ## load the Japanese election data
#' data(japan)
#' ## run the multinomial probit model with ordered preferences
#' res2 <- mnp(cbind(LDP, NFP, SKG, JCP) ~ gender + education + age, data = japan,
#'             verbose = TRUE)
#' ## summarize the results
#' summary(res2)
#' ## calculate the predicted probabilities for the 10th observation
#' ## averaging over 100 additional Monte Carlo draws given each of MCMC draw.
#' pre2 <- predict(res2, newdata = japan[10,], type = "prob", n.draws = 100,
#'                 verbose = TRUE)
#'
#' @export mnp
mnp <- function(formula, data = parent.frame(), choiceX = NULL,
                cXnames = NULL, base = NULL, latent = FALSE,
                invcdf = FALSE, trace = TRUE, n.draws = 5000, p.var = "Inf",
                p.df = n.dim+1, p.scale = 1, coef.start = 0,
                cov.start = 1, burnin = 0, thin = 0, verbose = FALSE) {
  call <- match.call()
  mf <- match.call(expand.dots = FALSE)
  mf$choiceX <- mf$cXnames <- mf$base <- mf$n.draws <- mf$latent <-
    mf$p.var <- mf$p.df <- mf$p.scale <- mf$coef.start <- mf$invcdf <-
      mf$trace <- mf$cov.start <- mf$verbose <- mf$burnin <- mf$thin <- NULL
  mf[[1]] <- as.name("model.frame")
  mf$na.action <- 'na.pass'
  mf <- eval.parent(mf)

  ## fix this parameter
  p.alpha0 <- 1

  ## obtaining Y
  tmp <- ymatrix.mnp(mf, base=base, extra=TRUE)
  Y <- tmp$Y
  MoP <- tmp$MoP
  lev <- tmp$lev
  base <- tmp$base
  p <- tmp$p
  n.dim <- p - 1
  if(verbose)
    cat("\nThe base category is `", base, "'.\n\n", sep="")
  if (p < 3)
    stop("The number of alternatives should be at least 3.")
  if(verbose)
    cat("The total number of alternatives is ", p, ".\n\n", sep="")
  if(verbose) {
    if (trace)
      cat("The trace restriction is used instead of the diagonal restriction.\n\n")
    else
      cat("The diagonal restriction is used instead of the trace restriction.\n\n")
  }

  ### obtaining X
  tmp <- xmatrix.mnp(formula, data=eval.parent(data),
                     choiceX=call$choiceX, cXnames=cXnames,
                     base=base, n.dim=n.dim, lev=lev, MoP=MoP,
                     verbose=verbose, extra=TRUE)
  X <- tmp$X
  coefnames <- tmp$coefnames
  n.cov <- ncol(X) / n.dim

  ## listwise deletion for X
  na.ind <- apply(is.na(X), 1, sum)
  if (ncol(Y) == 1)
    na.ind <- na.ind + is.na(Y)
  Y <- Y[na.ind==0,]
  X <- X[na.ind==0,]
  n.obs <- nrow(X)

  if (verbose) {
    cat("The dimension of beta is ", n.cov, ".\n\n", sep="")
    cat("The number of observations is ", n.obs, ".\n\n", sep="")
    if (sum(na.ind>0)>0) {
      if (sum(na.ind>0)==1)
        cat("The observation ", (1:length(na.ind))[na.ind>0], " is dropped due to missing values.\n\n", sep="")
      else {
        cat("The following ", sum(na.ind>0), " observations are dropped due to missing values:\n", sep="")
        cat((1:length(na.ind))[na.ind>0], "\n\n")
      }
    }
  }

  ## checking the prior for beta
  p.imp <- FALSE
  if (p.var == Inf) {
    p.imp <- TRUE
    p.prec <- diag(0, n.cov)
    if (verbose)
      cat("Improper prior will be used for beta.\n\n")
  }
  else if (is.matrix(p.var)) {
    if (ncol(p.var) != n.cov || nrow(p.var) != n.cov)
      stop("The dimension of `p.var' should be ", n.cov, " x ", n.cov, sep="")
    if (sum(sign(eigen(p.var)$values) < 1) > 0)
      stop("`p.var' must be positive definite.")
    p.prec <- solve(p.var)
  }
  else {
    p.var <- diag(p.var, n.cov)
    p.prec <- solve(p.var)
  }
  p.mean <- rep(0, n.cov)

  ## checking prior for Sigma
  p.df <- eval(p.df)
  if (length(p.df) > 1)
    stop("`p.df' must be a positive integer.")
  if (p.df < n.dim)
    stop(paste("`p.df' must be at least ", n.dim, ".", sep=""))
  if (abs(as.integer(p.df) - p.df) > 0)
    stop("`p.df' must be a positive integer.")
  if (!is.matrix(p.scale))
    p.scale <- diag(p.scale, n.dim)
  if (ncol(p.scale) != n.dim || nrow(p.scale) != n.dim)
    stop("`p.scale' must be ", n.dim, " x ", n.dim, sep="")
  if (sum(sign(eigen(p.scale)$values) < 1) > 0)
    stop("`p.scale' must be positive definite.")
  else if ((trace == FALSE) & (p.scale[1,1] != 1)) {
    p.scale[1,1] <- 1
    warning("p.scale[1,1] will be set to 1.")
  }
  Signames <- NULL
  for(j in 1:n.dim)
    for(k in 1:n.dim)
      if (j<=k)
        Signames <- c(Signames, paste(if(MoP) lev[j] else lev[j+1],
                                      ":", if(MoP) lev[k] else lev[k+1], sep=""))

  ## checking starting values
  if (length(coef.start) == 1)
    coef.start <- rep(coef.start, n.cov)
  else if (length(coef.start) != n.cov)
    stop(paste("The dimenstion of `coef.start' must be  ",
               n.cov, ".", sep=""))
  if (!is.matrix(cov.start)) {
    cov.start <- diag(n.dim)*cov.start
    if (!trace)
      cov.start[1,1] <- 1
  }
  else if (ncol(cov.start) != n.dim || nrow(cov.start) != n.dim)
    stop("The dimension of `cov.start' must be ", n.dim, " x ", n.dim, sep="")
  else if (sum(sign(eigen(cov.start)$values) < 1) > 0)
    stop("`cov.start' must be a positive definite matrix.")
  else if ((trace == FALSE) & (cov.start[1,1] != 1)) {
    cov.start[1,1] <- 1
    warning("cov.start[1,1] will be set to 1.")
  }

  ## checking thinnig and burnin intervals
  if (burnin < 0)
    stop("`burnin' should be a non-negative integer.")
  if (thin < 0)
    stop("`thin' should be a non-negative integer.")
  keep <- thin + 1

  ## running the algorithm
  if (latent)
    n.par <- n.cov + n.dim*(n.dim+1)/2 + n.dim*n.obs
  else
    n.par <- n.cov + n.dim*(n.dim+1)/2
  if(verbose)
    cat("Starting Gibbs sampler...\n")
  # recoding NA into -1
  Y[is.na(Y)] <- -1

  param <- .C("cMNPgibbs", as.integer(n.dim),
              as.integer(n.cov), as.integer(n.obs), as.integer(n.draws),
              as.double(p.mean), as.double(p.prec), as.integer(p.df),
              as.double(p.scale*p.alpha0), as.double(X), as.integer(Y),
              as.double(coef.start), as.double(cov.start),
              as.integer(p.imp), as.integer(invcdf),
              as.integer(burnin), as.integer(keep), as.integer(trace),
              as.integer(verbose), as.integer(MoP), as.integer(latent),
              pdStore = double(n.par*floor((n.draws-burnin)/keep)),
              PACKAGE="MNP")$pdStore
  param <- matrix(param, ncol = n.par,
                  nrow = floor((n.draws-burnin)/keep), byrow=TRUE)
  if (latent) {
    W <- array(as.vector(t(param[,(n.par-n.dim*n.obs+1):n.par])),
               dim = c(n.dim, n.obs, floor((n.draws-burnin)/keep)),
               dimnames = list(lev[!(lev %in% base)], rownames(Y), NULL))
    param <- param[,1:(n.par-n.dim*n.obs)]
    }
  else
    W <- NULL
  colnames(param) <- c(coefnames, Signames)

  ##recoding -1 back into NA
  Y[Y==-1] <- NA

  ## returning the object
  res <- list(param = param, x = X, y = Y, w = W, call = call, alt = lev,
              n.alt = p, base = base, invcdf = invcdf, trace = trace,
              p.mean = if(p.imp) NULL else p.mean, p.var = p.var,
              p.df = p.df, p.scale = p.scale, burnin = burnin, thin = thin)
  class(res) <- "mnp"
  return(res)
}

xmatrix.mnp <- function(formula, data = parent.frame(), choiceX=NULL,
                        cXnames=NULL, base=NULL, n.dim, lev,
                        MoP=FALSE, verbose=FALSE, extra=FALSE) {
  call <- match.call()
  mf <- match.call(expand.dots = FALSE)
  mf$choiceX <- mf$cXnames <- mf$base <- mf$n.dim <- mf$lev <-
    mf$MoP <- mf$verbose <- mf$extra <- NULL  
  
  ## get variables
  mf[[1]] <- as.name("model.frame.default")
  mf$na.action <- 'na.pass'
  mf <- eval.parent(mf)
  Terms <- attr(mf, "terms")
  X <- stats::model.matrix.default(Terms, mf)
  xvars <- as.character(attr(Terms, "variables"))[-1]
  if ((yvar <- attr(Terms, "response")) > 0)
    xvars <- xvars[-yvar]
  xlev <- if (length(xvars) > 0) {
    xlev <- lapply(mf[xvars], levels)
    xlev[!sapply(xlev, is.null)]
  }
  p <- n.dim + 1
  n.obs <- nrow(X)
  n.cov <- ncol(X)
  
  ## expanding X
  allvnames <- Xnew <- NULL
  if (ncol(X) > 0) {
    Xcnames <- colnames(X)
    for (i in 1:n.cov) {
      Xv <- X[, Xcnames[i]]
      Xtmp <- varnames <- NULL
      for (j in 1:n.dim) {
        allvnames <- c(allvnames, paste(Xcnames[i], ":", if(MoP)
                                        lev[j] else lev[j+1], sep=""))
        for (k in 1:n.dim)
          varnames <- c(varnames, paste(Xcnames[i], ":", if(MoP) lev[j]
          else lev[j+1], sep=""))
        tmp <- matrix(0, nrow = n.obs, ncol = n.dim)
        tmp[, j] <- Xv
        Xtmp <- cbind(Xtmp, tmp)
      }
      colnames(Xtmp) <- varnames
      Xnew <- cbind(Xnew, Xtmp)
    }
  }
  
  ## checking and adding choice-specific variables
  if (!is.null(choiceX)) {
    cX <- eval(choiceX, data)
    cXn <- unique(names(cX))
    if (sum(is.na(pmatch(cXn, lev))) > 0)
      stop(paste("Error: Invalid input for `choiceX.'\n Some variables do not exist."))
    if(MoP) 
      xbase <- as.matrix(cX[[lev[p]]])
    else if (is.na(pmatch(base, cXn)))
      xbase <- NULL
    else
      xbase <- as.matrix(cX[[base]])
    if (length(cXn) < n.dim)
      stop(paste("Error: Invalid input for `choiceX.'\n You must specify the choice-specific varaibles at least for all non-base categories."))
    if (!is.null(xbase) && length(cXn) != p)
      stop(paste("Error: Invalid input for `choiceX.'\n You must specify the choice-specific variables at least for all non-base categories."))
    if(!is.null(xbase) && verbose)
      cat("The choice-specific variables of the base category are subtracted from the corresponding variables of the non-base categories.\n\n")
    for (i in 1:length(cXnames)) 
      for (j in 1:n.dim) {
        if (length(cXnames) != ncol(as.matrix(cX[[if(MoP) lev[j] else lev[j+1]]])))
            stop(paste("Error: The number of variables in `choiceX' and `cXnames' does not match."))  
        tmp <- matrix(as.matrix(cX[[if(MoP) lev[j] else lev[j+1]]])[,i], ncol=1)
        if (!is.null(xbase)) 
          tmp <- tmp - xbase[,i]
        colnames(tmp) <- paste(cXnames[i], ":", if(MoP) lev[j] else lev[j+1], sep="") 
        Xnew <- cbind(Xnew, tmp)
      }
  }
  if(extra)
    return(list(X=Xnew, coefnames=c(allvnames, cXnames)))
  else
    return(Xnew)
}

#' Extract Multinomial Probit Model Covariance Matrix
#'
#' \code{vcov.mnp} is a function which extracts the posterior draws of
#' covariance matrix from objects returned by \code{mnp}.
#'
#'
#' @param object An output object from \code{mnp}.
#' @param subset A scalar or a numerical vector specifying the row number(s) of
#' \code{param} in the output object from \code{mnp}. If specified, the
#' posterior draws of covariance matrix for those rows are extracted. The
#' default is \code{NULL} where all the posterior draws are extracted.
#' @param ... further arguments passed to or from other methods.
#' @return When a numerical vector or \code{NULL} is specified for
#' \code{subset} argument, \code{vcov.mnp} returns a three dimensional array
#' where the third dimension indexes posterior draws. When a scalar is
#' specified for \code{subset} arugment, \code{vcov.mnp} returns a matrix.
#' @author Kosuke Imai, Department of Government and Department of Statistics, Harvard University
#' \email{imai@@harvard.edu}
#' @seealso \code{mnp}, \code{coef.mnp};
#' @keywords methods
#' @exportS3Method vcov mnp
vcov.mnp <- function(object, subset = NULL, ...) {
  if (is.null(subset))
    subset <- 1:nrow(object$param)
  else if (max(subset) > nrow(object$param))
    stop(paste("invalid input for `subset.' only", nrow(object$param), "draws are stored."))

  p <- object$n.alt
  n <- length(subset)
  Sigma <- array(0, c(p-1, p-1, n))
  param <- object$param
  n.cov <- ncol(param) - p*(p-1)/2
  cov <- param[,(n.cov+1):ncol(param)]
  for (i in 1:n) {
    count <- 1
    for (j in 1:(p-1)) {
      Sigma[j,j:(p-1),i] <- cov[subset[i],count:(count+p-j-1)]
      count <- count + p - j
    }
    diag(Sigma[,,i]) <- diag(Sigma[,,i]/2)
    Sigma[,,i] <- Sigma[,,i] + t(Sigma[,,i])
  }
  tmp <- list()
  tmp[[1]] <- tmp[[2]] <- object$alt[-pmatch(object$base, object$alt)]
  tmp[[3]] <- as.character(subset)
  dimnames(Sigma) <- tmp
  if (n > 1)
    return(Sigma)
  else
    return(Sigma[,,1])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.mnp.R
\name{print.summary.mnp}
\alias{print.summary.mnp}
\title{Print the summary of the results for the Multinomial Probit Models}
\usage{
\method{print}{summary.mnp}(x, digits = max(3, getOption("digits") - 3), ...)
}
\arguments{
\item{x}{An object of class \code{summary.mnp}.}

\item{digits}{the number of significant digits to use when printing.}

\item{...}{further arguments passed to or from other methods.}
}
\description{
\code{summary} print method for class \code{mnp}.
}
\seealso{
\code{mnp}
}
\author{
Kosuke Imai, Department of Politics, Princeton University
\email{kimai@Princeton.Edu}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.mnp.R
\name{coef.mnp}
\alias{coef.mnp}
\title{Extract Multinomial Probit Model Coefficients}
\usage{
\method{coef}{mnp}(object, subset = NULL, ...)
}
\arguments{
\item{object}{An output object from \code{mnp}.}

\item{subset}{A scalar or a numerical vector specifying the row number(s) of
\code{param} in the output object from \code{mnp}. If specified, the
posterior draws of coefficients for those rows are extracted. The default is
\code{NULL} where all the posterior draws are extracted.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
\code{coef.mnp} returns a matrix (when a numerical vector or
\code{NULL} is specified for \code{subset} argument) or a vector (when a
scalar is specified for \code{subset} arugment) of multinomila probit model
coefficients.
}
\description{
\code{coef.mnp} is a function which extracts multinomial probit model
coefficients from ojbects returned by \code{mnp}. \code{coefficients.mnp} is
an alias for it. \code{coef} method for class \code{mnp}.
}
\seealso{
\code{mnp}, \code{vcov.mnp};
}
\author{
Kosuke Imai, Department of Government and Department of Statistics, Harvard University
\email{kimai@Princeton.Edu}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnp.R
\name{mnp}
\alias{mnp}
\alias{MNP}
\title{Fitting the Multinomial Probit Model via Markov chain Monte Carlo}
\usage{
mnp(
  formula,
  data = parent.frame(),
  choiceX = NULL,
  cXnames = NULL,
  base = NULL,
  latent = FALSE,
  invcdf = FALSE,
  trace = TRUE,
  n.draws = 5000,
  p.var = "Inf",
  p.df = n.dim + 1,
  p.scale = 1,
  coef.start = 0,
  cov.start = 1,
  burnin = 0,
  thin = 0,
  verbose = FALSE
)
}
\arguments{
\item{formula}{A symbolic description of the model to be fit specifying the
response variable and covariates. The formula should not include the
choice-specific covariates. Details and specific examples are given below.}

\item{data}{An optional data frame in which to interpret the variables in
\code{formula} and \code{choiceX}. The default is the environment in which
\code{mnp} is called.}

\item{choiceX}{An optional list containing a matrix of choice-specific
covariates for each category. Details and examples are provided below.}

\item{cXnames}{A vector of the names for the choice-specific covariates
specified in \code{choiceX}. The details and examples are provided below.}

\item{base}{The name of the base category. For the standard multinomial
probit model, the default is the lowest level of the response variable. For
the multinomial probit model with ordered preferences, the default base
category is the last column in the matrix of response variables.}

\item{latent}{logical. If \code{TRUE}, then the latent variable W will be
returned. See Imai and van Dyk (2005) for the notation. The default is
\code{FALSE}.}

\item{invcdf}{logical. If \code{TRUE}, then the inverse cdf method is used
for truncated normal sampling. If \code{FALSE}, then the rejection sampling
method is used. The default is \code{FALSE}.}

\item{trace}{logical. If \code{TRUE}, then the trace of the variance
covariance matrix is set to a constant (here, it is equal to \code{n.dim})
instead of setting its first diagonal element to 1.  The former avoids the
arbitrariness of fixing one particular diagonal element in order to achieve
identification (see Burgette and Nordheim, 2009).}

\item{n.draws}{A positive integer. The number of MCMC draws. The default is
\code{5000}.}

\item{p.var}{A positive definite matrix. The prior variance of the
coefficients.  A scalar input can set the prior variance to the diagonal
matrix whose diagonal element is equal to that value. The default is
\code{"Inf"}, which represents an improper noninformative prior distribution
on the coefficients.}

\item{p.df}{A positive integer greater than \code{n.dim-1}. The prior
degrees of freedom parameter for the covariance matrix. The default is
\code{n.dim+1}, which is equal to the total number of alternatives.}

\item{p.scale}{A positive definite matrix.  When \code{trace = FALSE}, its
first diagonal element is set to \code{1} if it is not equal to 1 already.
The prior scale matrix for the covariance matrix. A scalar input can be used
to set the scale matrix to a diagonal matrix with diagonal elements equal to
the scalar input value. The default is \code{1}.}

\item{coef.start}{A vector. The starting values for the coefficients.  A
scalar input sets the starting values for all the coefficients equal to that
value.  The default is \code{0}.}

\item{cov.start}{A positive definite matrix. When \code{trace = FALSE}, its
first diagonal element is set to \code{1} if it is not equal to 1 already.
The starting values for the covariance matrix. A scalar input can be used to
set the starting value to a diagonal matrix with diagonal elements equal to
the scalar input value. The default is \code{1}.}

\item{burnin}{A positive integer. The burnin interval for the Markov chain;
i.e., the number of initial Gibbs draws that should not be stored. The
default is \code{0}.}

\item{thin}{A positive integer. The thinning interval for the Markov chain;
i.e., the number of Gibbs draws between the recorded values that are
skipped. The default is \code{0}.}

\item{verbose}{logical. If \code{TRUE}, helpful messages along with a
progress report of the Gibbs sampling are printed on the screen. The default
is \code{FALSE}.}
}
\value{
An object of class \code{mnp} containing the following elements:
\item{param}{A matrix of the Gibbs draws for each parameter; i.e., the
coefficients and covariance matrix. For the covariance matrix, the elements
on or above the diagonal are returned.  }
\item{call}{The matched call.}
\item{x}{The matrix of covariates.}
\item{y}{The vector or matrix of the
response variable.}
\item{w}{The three dimensional array of the latent
variable, W. The first dimension represents the alternatives, and the second
dimension indexes the observations. The third dimension represents the Gibbs
draws. Note that the latent variable for the base category is set to 0, and
therefore omitted from the output.}
\item{alt}{The names of alternatives.}
\item{n.alt}{The total number of alternatives.}
\item{base}{The base
category used for fitting.}
\item{invcdf}{The value of
\code{invcdf}.}
\item{p.var}{The prior variance for the coefficients.}
\item{p.df}{The prior
degrees of freedom parameter for the covariance matrix.}
\item{p.scale}{The
prior scale matrix for the covariance matrix.}
\item{burnin}{The number of
initial burnin draws.}
\item{thin}{The thinning interval.}
}
\description{
\code{mnp} is used to fit (Bayesian) multinomial probit model via Markov
chain Monte Carlo.  \code{mnp} can also fit the model with different choice
sets for each observation, and complete or partial ordering of all the
available alternatives. The computation uses the efficient marginal data
augmentation algorithm that is developed by Imai and van Dyk (2005a).
}
\details{
To fit the multinomial probit model when only the most preferred choice is
observed, use the syntax for the formula, \code{y ~ x1 + x2}, where \code{y}
is a factor variable indicating the most preferred choice and \code{x1} and
\code{x2} are individual-specific covariates. The interactions of
individual-specific variables with each of the choice indicator variables
will be fit.

To specify choice-specific covariates, use the syntax,
\code{choiceX=list(A=cbind(z1, z2), B=cbind(z3, z4), C=cbind(z5, z6))},
where \code{A}, \code{B}, and \code{C} represent the choice names of the
response variable, and \code{z1} and \code{z2} are each vectors of length
\eqn{n} that record the values of the two choice-specific covariates for
each individual for choice A, likewise for \code{z3}, \eqn{\ldots},
\code{z6}. The corresponding variable names via \code{cXnames=c("price",
"quantity")} need to be specified, where \code{price} refers to the
coefficient name for \code{z1}, \code{z3}, and \code{z5}, and
\code{quantity} refers to that for \code{z2}, \code{z4}, and \code{z6}.

If the choice set varies from one observation to another, use the syntax,
\code{cbind(y1, y2, y3) ~ x1 + x2}, in the case of a three choice problem,
and indicate unavailable alternatives by \code{NA}. If only the most
preferred choice is observed, \code{y1}, \code{y2}, and \code{y3} are
indicator variables that take on the value one for individuals who prefer
that choice and zero otherwise. The last column of the response matrix,
\code{y3} in this particular example syntax, is used as the base category.

To fit the multinomial probit model when the complete or partial ordering of
the available alternatives is recorded, use the same syntax as when the
choice set varies (i.e., \code{cbind(y1, y2, y3, y4) ~ x1 + x2}). For each
observation, all the available alternatives in the response variables should
be numerically ordered in terms of preferences such as \code{1 2 2 3}. Ties
are allowed. The missing values in the response variable should be denoted
by \code{NA}. The software will impute these missing values using the
specified covariates. The resulting uncertainty estimates of the parameters
will properly reflect the amount of missing data. For example, we expect the
standard errors to be larger when there is more missing data.
}
\examples{

###
### NOTE: this example is not fully analyzed. In particular, the
### convergence has not been assessed. A full analysis of these data
### sets appear in Imai and van Dyk (2005b).
###

## load the detergent data
data(detergent)
## run the standard multinomial probit model with intercepts and the price
res1 <- mnp(choice ~ 1, choiceX = list(Surf=SurfPrice, Tide=TidePrice,
                                       Wisk=WiskPrice, EraPlus=EraPlusPrice,
                                       Solo=SoloPrice, All=AllPrice),
            cXnames = "price", data = detergent, n.draws = 100, burnin = 10,
            thin = 3, verbose = TRUE)
## summarize the results
summary(res1)
## calculate the quantities of interest for the first 3 observations
pre1 <- predict(res1, newdata = detergent[1:3,])

## load the Japanese election data
data(japan)
## run the multinomial probit model with ordered preferences
res2 <- mnp(cbind(LDP, NFP, SKG, JCP) ~ gender + education + age, data = japan,
            verbose = TRUE)
## summarize the results
summary(res2)
## calculate the predicted probabilities for the 10th observation
## averaging over 100 additional Monte Carlo draws given each of MCMC draw.
pre2 <- predict(res2, newdata = japan[10,], type = "prob", n.draws = 100,
                verbose = TRUE)

}
\references{
Imai, Kosuke and David A. van Dyk. (2005a) \dQuote{A Bayesian
Analysis of the Multinomial Probit Model Using the Marginal Data
Augmentation,} \emph{Journal of Econometrics}, Vol. 124, No. 2 (February),
pp.311-334.

Imai, Kosuke and David A. van Dyk. (2005b) \dQuote{MNP: R Package for
Fitting the Multinomial Probit Models,} \emph{Journal of Statistical
Software}, Vol. 14, No. 3 (May), pp.1-32.

Burgette, L.F. and E.V. Nordheim. (2009).  \dQuote{An alternate identifying
restriction for the Bayesian multinomial probit model,} \emph{Technical
report}, Department of Statistics, University of Wisconsin, Madison.
}
\seealso{
\code{coef.mnp}, \code{vcov.mnp}, \code{predict.mnp},
\code{summary.mnp};
}
\author{
Kosuke Imai, Department of Government and Department of Statistics, Harvard University
\email{imai@Harvard.Edu}, \url{https://imai.fas.harvard.edu}; David A. van
Dyk, Statistics Section, Department of Mathematics, Imperial College London.
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov.mnp.R
\name{vcov.mnp}
\alias{vcov.mnp}
\title{Extract Multinomial Probit Model Covariance Matrix}
\usage{
\method{vcov}{mnp}(object, subset = NULL, ...)
}
\arguments{
\item{object}{An output object from \code{mnp}.}

\item{subset}{A scalar or a numerical vector specifying the row number(s) of
\code{param} in the output object from \code{mnp}. If specified, the
posterior draws of covariance matrix for those rows are extracted. The
default is \code{NULL} where all the posterior draws are extracted.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
When a numerical vector or \code{NULL} is specified for
\code{subset} argument, \code{vcov.mnp} returns a three dimensional array
where the third dimension indexes posterior draws. When a scalar is
specified for \code{subset} arugment, \code{vcov.mnp} returns a matrix.
}
\description{
\code{vcov.mnp} is a function which extracts the posterior draws of
covariance matrix from objects returned by \code{mnp}.
}
\seealso{
\code{mnp}, \code{coef.mnp};
}
\author{
Kosuke Imai, Department of Government and Department of Statistics, Harvard University
\email{imai@harvard.edu}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.mnp.R
\name{summary.mnp}
\alias{summary.mnp}
\title{Summarizing the results for the Multinomial Probit Models}
\usage{
\method{summary}{mnp}(object, CI = c(2.5, 97.5), ...)
}
\arguments{
\item{object}{An output object from \code{mnp}.}

\item{CI}{A 2 dimensional vector of lower and upper bounds for the credible
intervals used to summarize the results. The default is the equal tail 95
percent credible interval.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
\code{summary.mnp} yields an object of class \code{summary.mnp}
containing the following elements: \item{call}{The call from \code{mnp}.}
\item{n.alt}{The total number of alternatives.} \item{base}{The base
category used for fitting.} \item{n.obs}{The number of observations.}
\item{n.param}{The number of estimated parameters.} \item{n.draws}{The
number of Gibbs draws used for the summary.} \item{coef.table}{The summary
of the posterior distribution of the coefficients. } \item{cov.table}{The
summary of the posterior distribution of the covariance matrix.} This object
can be printed by \code{print.summary.mnp}
}
\description{
\code{summary} method for class \code{mnp}.
}
\seealso{
\code{mnp}
}
\author{
Kosuke Imai, Department of Government and Department of Statistics, Harvard University
\email{imai@Harvard.Edu}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/japan.R
\docType{data}
\name{japan}
\alias{japan}
\title{Voters' Preferences of Political Parties in Japan (1995)}
\format{
A data frame containing the following 7 variables for 418
observations.

\tabular{llll}{ LDP \tab numeric \tab preference for Liberal Democratic
Party \tab 0 - 100 \cr NFP \tab numeric \tab preference for New Frontier
Party \tab 0 - 100 \cr SKG \tab numeric \tab preference for Sakigake \tab 0
- 100 \cr JCP \tab numeric \tab preference for Japanese Communist Party \tab
0 - 100 \cr gender \tab factor \tab gender of each voter \tab \code{male} or
\code{female} \cr education \tab numeric \tab levels of education for each
voter \tab \cr age \tab numeric \tab age of each voter \tab }
}
\description{
This dataset gives voters' preferences of political parties in Japan on the
0 (least preferred) - 100 (most preferred) scale.  It is based on the 1995
survey data of 418 individual voters.  The data also include the sex,
education level, and age of the voters. The survey allowed voters to choose
among four parties: Liberal Democratic Party (LDP), New Frontier Party
(NFP), Sakigake (SKG), and Japanese Communist Party (JCP).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detergent.R
\docType{data}
\name{detergent}
\alias{detergent}
\title{Detergent Brand Choice}
\format{
A data frame containing the following 7 variables and 2657
observations.

\tabular{lll}{ choice \tab factor \tab a brand chosen by each household\cr
TidePrice \tab numeric \tab log price of Tide\cr WiskPrice \tab numeric \tab
log price of Wisk\cr EraPlusPrice \tab numeric \tab log price of EraPlus\cr
SurfPrice \tab numeric \tab log price of Surf\cr SoloPrice \tab numeric \tab
log price of Solo\cr AllPrice \tab numeric \tab log price of All\cr }
}
\description{
This dataset gives the laundry detergent brand choice by households and the
price of each brand.
}
\references{
Chintagunta, P. K. and Prasad, A. R. (1998) \dQuote{An Empirical
Investigation of the `Dynamic McFadden' Model of Purchase Timing and Brand
Choice: Implications for Market Structure}. \emph{Journal of Business and
Economic Statistics} vol. 16 no. 1 pp.2-12.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.mnp.R
\name{predict.mnp}
\alias{predict.mnp}
\title{Posterior Prediction under the Bayesian Multinomial Probit Models}
\usage{
\method{predict}{mnp}(
  object,
  newdata = NULL,
  newdraw = NULL,
  n.draws = 1,
  type = c("prob", "choice", "order"),
  verbose = FALSE,
  ...
)
}
\arguments{
\item{object}{An output object from \code{mnp}.}

\item{newdata}{An optional data frame containing the values of the predictor
variables. Predictions for multiple values of the predictor variables can be
made simultaneously if \code{newdata} has multiple rows. The default is the
original data frame used for fitting the model.}

\item{newdraw}{An optional matrix of MCMC draws to be used for posterior
predictions. The default is the original MCMC draws stored in \code{object}.}

\item{n.draws}{The number of additional Monte Carlo draws given each MCMC
draw of coefficients and covariance matrix. The specified number of latent
variables will be sampled from the multivariate normal distribution, and the
quantities of interest will be calculated by averaging over these draws.
This will be particularly useful calculating the uncertainty of predicted
probabilities. The default is \code{1}.}

\item{type}{The type of posterior predictions required. There are four
options: \code{type = "prob"} returns the predictive probabilities of being
the most preferred choice among the choice set.  \code{type = "choice"}
returns the Monte Carlo sample of the most preferred choice, and \code{type
= "order"} returns the Monte Carlo sample of the ordered preferences,}

\item{verbose}{logical. If \code{TRUE}, helpful messages along with a
progress report on the Monte Carlo sampling from the posterior predictive
distributions are printed on the screen. The default is \code{FALSE}.}

\item{...}{additional arguments passed to other methods.}
}
\value{
\code{predict.mnp} yields a list of class
\code{predict.mnp} containing at least one of the following elements:
\item{o}{A three dimensional array of the Monte Carlo sample from the posterior predictive
distribution of the ordered preferences. The first dimension corresponds to
the rows of \code{newdata} (or the original data set if \code{newdata} is
left unspecified), the second dimension corresponds to the alternatives in
the choice set, and the third dimension indexes the Monte Carlo sample. If
\code{n.draws} is greater than 1, then each entry will be an average over
these additional draws.  }
\item{p}{A two or three dimensional array of the
posterior predictive probabilities for each alternative in the choice set
being most preferred. The first demension corresponds to the rows of
\code{newdata} (or the original data set if \code{newdata} is left
unspecified), the second dimension corresponds to the alternatives in the
choice set, and the third dimension (if it exists) indexes the Monte Carlo
sample. If \code{n.draws} is greater than 1, then the third diemsion exists
and indexes the Monte Carlo sample.  }
\item{y}{A matrix of the Monte Carlo
sample from the posterior predictive distribution of the most preferred
choice. The first dimension correspond to the rows of \code{newdata} (or the
original data set if \code{newdata} is left unspecified), and the second
dimension indexes the Monte Carlo sample. \code{n.draws} will be set to 1
when computing this quantity of interest.  }
\item{x}{A matrix of covariates used for prediction}
}
\description{
Obtains posterior predictions under a fitted (Bayesian) multinomial probit
model. \code{predict} method for class \code{mnp}.
}
\details{
The posterior predictive values are computed using the Monte Carlo sample
stored in the \code{mnp} output (or other sample if \code{newdraw} is
specified). Given each Monte Carlo sample of the parameters and each vector
of predictor variables, we sample the vector-valued latent variable from the
appropriate multivariate Normal distribution. Then, using the sampled
predictive values of the latent variable, we construct the most preferred
choice as well as the ordered preferences. Averaging over the Monte Carlo
sample of the preferred choice, we obtain the predictive probabilities of
each choice being most preferred given the values of the predictor
variables.  Since the predictive values are computed via Monte Carlo
simulations, each run may produce somewhat different values. The computation
may be slow if predictions with many values of the predictor variables are
required and/or if a large Monte Carlo sample of the model parameters is
used. In either case, setting \code{verbose = TRUE} may be helpful in
monitoring the progress of the code.
}
\seealso{
\code{mnp}
}
\author{
Kosuke Imai, Department of Government and Department of Statistics, Harvard University
\email{imai@Harvard.Edu}
}
\keyword{methods}
